@@ -4,12 +4,10 @@
 # and Compo 1998
 # and 'Identification of Chirps with Continuous Wavelet Transform'
 # from Carmona,Hwang and Torresani 1995
-#
-# Version 0.7 March 2020, Gregor Moenke (gregor.moenke@embl.de)
 ###########################################################################
 
 import numpy as np
-from numpy.fft import rfft, rfftfreq, fft
+from numpy.fft import rfft, rfftfreq
 from numpy.random import uniform, randn, randint, choice
 from numpy import pi
 from scipy.signal import savgol_filter
@@ -18,12 +16,15 @@ import pandas as pd
 # global variables
 # -----------------------------------------------------------
 omega0 = 2 * pi  # central frequency of the mother wavelet
-xi2_95 = 5.99
-xi2_99 = 9.21
+chi2_95 = 5.99    # 0.05 confidence interval 
+chi2_99 = 9.21    # 0.01 confidence interval 
 
 # clip Wavelets at 1/peak_fraction envelope
-peak_fraction = 1e6
+peak_fraction = 1e8
 clip_support = True
+
+# max sinc length, increases performance for long signals
+M_max = 2000
 # -----------------------------------------------------------
 
 
@@ -31,31 +32,32 @@ def compute_spectrum(signal, dt, periods):
 
     """
 
-        Computes the Wavelet spectrum for a given *signal* for the given *periods*
+    Computes the Wavelet spectrum for a given *signal* 
+    for the given *periods*.
 
-        Parameters
-        ----------
+    Parameters
+    ----------
 
-        signal  : a sequence
-                  the time-series to be analyzed, 
-                  you might want to detrend beforehand!
+    signal  : a sequence
+              the time-series to be analyzed, 
+              you might want to detrend beforehand!
 
-        dt      : the sampling interval scaled to desired time units
+    dt      : the sampling interval scaled to desired time units
 
-        periods : the list of periods to compute the Wavelet spectrum for, 
-                  must have same units as dt!
+    periods : the list of periods to compute the Wavelet spectrum for, 
+              must have same units as dt!
 
 
-        Returns
-        -------
+    Returns
+    -------
 
-        modulus : 2d ndarray of reals, 
-                  the Wavelet power spectrum normalized by signal variance
+    modulus : 2d ndarray of reals, 
+              the Wavelet power spectrum normalized by signal variance
 
-        wlet : 2d complex ndarray, 
-               the Wavelet transform with dimensions len(periods) x len(signal) 
-        
-        """
+    transform : 2d complex ndarray, 
+           the Wavelet transform with dimensions len(periods) x len(signal) 
+
+    """
 
     if periods[0] < 2 * dt:
         print()
@@ -69,8 +71,6 @@ def compute_spectrum(signal, dt, periods):
     dt = float(dt)
     sfreq = 1 / dt  # the sampling frequency
 
-    Nt = len(signal)  # number of time points
-
     # --------------------------------------------
     scales = scales_from_periods(periods, sfreq, omega0)
     # --------------------------------------------
@@ -85,11 +85,11 @@ def compute_spectrum(signal, dt, periods):
         print("proceeding anyways...")
 
     Morlet = mk_Morlet(omega0)
-    wlet = CWT(signal, Morlet, scales)  # complex wavelet transform
+    transform = CWT(signal, Morlet, scales)  # complex wavelet transform
     sig2 = np.var(signal)  # white noise has then mean power of one
-    modulus = np.abs(wlet) ** 2 / sig2  # normalize with variance of signal
+    modulus = np.abs(transform) ** 2 / sig2  # normalize with variance of signal
 
-    return modulus, wlet
+    return modulus, transform
 
 
 def get_maxRidge_ys(modulus):
@@ -103,7 +103,8 @@ def get_maxRidge_ys(modulus):
     ----------
 
     modulus : 2d ndarray of reals, 
-              the Wavelet power spectrum (periods x time) normalized by signal variance
+              the Wavelet power spectrum (periods x time) 
+              normalized by signal variance
 
     Returns
     -------
@@ -118,7 +119,13 @@ def get_maxRidge_ys(modulus):
 
 
 def eval_ridge(
-    ridge_y, wlet, signal, periods, tvec, power_thresh=0, smoothing_wsize=None
+        ridge_y,
+        transform,
+        signal,
+        periods,
+        tvec,
+        power_thresh=0,
+        smoothing_wsize=None
 ):
 
     """
@@ -132,7 +139,7 @@ def eval_ridge(
     ridge_y :  sequence of indices, has the length of the time axis
                of the spectrum, e.i. the y-coordinates of a ridge
 
-    wlet :     2d complex ndarray, holds the complex Wavelet transform 
+    transform :     2d complex ndarray, holds the complex Wavelet transform 
                with dimensions len(periods) x len(signal)
 
     signal :   1d ndarray, the original signal to be analyzed, 
@@ -170,7 +177,7 @@ def eval_ridge(
     """
 
     sigma2 = np.var(signal)
-    modulus = np.abs(wlet) ** 2 / sigma2  # normalize with variance of signal
+    modulus = np.abs(transform) ** 2 / sigma2  # normalize with variance of signal
 
     # calculate here to minimize arguments needed
     dt = tvec[1] - tvec[0]
@@ -181,7 +188,7 @@ def eval_ridge(
     index = np.arange(Nt) 
     
     ridge_per = periods[ridge_y]
-    ridge_z = wlet[ridge_y, np.arange(Nt)]  # picking the right t-y values !
+    ridge_z = transform[ridge_y, np.arange(Nt)]  # picking the right t-y values !
 
     ridge_power = modulus[ridge_y, np.arange(Nt)]
 
@@ -202,7 +209,7 @@ def eval_ridge(
         # sanitize ridge smoothing window size
         if Ntt < smoothing_wsize:
             # need an odd window size
-            smoothing = Ntt if Ntt % 2 == 1 else Ntt - 1
+            smoothing_wsize = Ntt if Ntt % 2 == 1 else Ntt - 1
 
         # print('inds:', inds)
         # smoothed maximum estimate of the whole ridge..
@@ -235,8 +242,66 @@ def eval_ridge(
 
     return ridge_data
 
+
+# --- confidence intervals ---
+
+def get_significant_regions(modulus,
+                            empirical_background,
+                            confidence = chi2_95):
+
+    '''
+    Given an (empirical) background Fourier power spectrum,
+    returns a boolean mask indicating areas of significant 
+    oscillations within the wavelet power spectrum for 
+    the desired confidence interval.
+
+    Parameters
+    ----------
+
+    modulus : 2d ndarray of reals, 
+              the wavelet power spectrum normalized by signal variance
+
+    empirical_background: 1d sequence, Fourier estimate of the background. 
+                          Must hold the powers at exactly the periods used for
+                          the wavelet analysis!
+
+    confidence : float, the Chi-squared value at the desired confidence level.
+                 Defaults to the 95% confidence interval.
+
+    Returns
+    -------
+
+    sign_regions : 2d boolean ndarray with the shape of *modulus*,
+                   is True for every significant point of the
+                   power spectrum
+
+    '''
+        
+    # every period needs a background power value
+    # (constant 1 for white noise for examle)
+    if not len(empirical_background) == modulus.shape[0]:
+        raise ValueError("Empirical background doesn't fit"
+                         " to wavelet spectrum!")
+
+    # remove DOF factor from chi2
+    conf = confidence / 2.0
+
+    # rescale every column along the period axis
+    # with the assumed 0-model spectrum    
+    scaled_mod = np.zeros(modulus.shape)
+    for i, col in enumerate(modulus.T):
+        scaled_mod[:, i] = col / empirical_background
+
+    # return boolean mask of the power spectrum
+    # indicating significant oscillations
+    sign_regions = scaled_mod > conf
+    
+    return sign_regions
+
+
 # --- COI business ---
 
+
 def get_COI_branches(time_vector):
 
     """
@@ -262,6 +327,7 @@ def get_COI_branches(time_vector):
 
     return np.r_[left, right], np.r_[left_t, right_t]
 
+
 def find_COI_crossing(rd):
 
     """
@@ -443,7 +509,7 @@ def smooth(x, window_len=11, window="flat", data=None):
     if window_len % 2 == 0:
         raise ValueError("window_len should be odd")
 
-    if not window in ["flat", "extern"]:
+    if window not in ["flat", "extern"]:
         raise ValueError("Window is none of 'flat' or 'extern'")
 
     s = np.r_[x[window_len - 1 : 0 : -1], x, x[-1:-window_len:-1]]
@@ -470,13 +536,18 @@ def sinc_filter(M, f_c=0.2):
 
     """
 
-    # not very effective, but should be get called only once per convolution
+    # not very effective, but should be gets called only once per convolution
 
+    # limit the filter's maximal size
+    if M > M_max:
+        M = M_max
+    
     assert M % 2 == 0, "M must be even!"
     res = []
 
     for x in np.arange(0, M + 1):
 
+        # this is the non-vectorizable part
         if x == M / 2:
             res.append(2 * pi * f_c)
             continue
@@ -549,9 +620,13 @@ def sliding_window_amplitude(signal, window_size, dt, SGsmooth=True):
     # get the underlying array
     vector = np.array(signal)
 
+    if window_size > (len(signal) - 1) * dt:
+        window_size = (len(signal) - 1) * dt
+        print(f'Warning, setting window_size to {window_size}!')
+    
     # window size in sampling interval units
     window_size = int(window_size / dt)
-    
+        
     # has to be odd
     if window_size % 2 != 1:
         window_size = window_size + 1
@@ -596,14 +671,17 @@ def normalize_with_envelope(dsignal, window_size, dt):
     ----------
 
     dsignal : ndarray, the (detrended) signal
-    window_size : int, the window size in sampling time units
+    window_size : int, the window size in time units, e.g. 17 minutes
     dt : float, the sampling interval 
     """
 
     # mean subtraction
     signal = dsignal - dsignal.mean()
 
-    
+    if window_size > (len(signal) - 1) * dt:
+        window_size = (len(signal) - 1) * dt
+        print(f'Warning, setting window_size to {window_size}!')
+              
     # ampl. normalization
     env = sliding_window_amplitude(signal, window_size, dt)
     ANsignal = 1 / env * dsignal
