@@ -1,58 +1,57 @@
-import sys, os
+import os
 import numpy as np
 
 from PyQt5.QtWidgets import (
     QCheckBox,
+    QMessageBox,
     QTableView,
     QComboBox,
     QFileDialog,
-    QAction,
     QMainWindow,
-    QApplication,
     QLabel,
     QLineEdit,
     QPushButton,
-    QMessageBox,
-    QSizePolicy,
     QWidget,
     QVBoxLayout,
     QHBoxLayout,
-    QDialog,
     QGroupBox,
     QFormLayout,
     QGridLayout,
     QTabWidget,
-    QTableWidget,
 )
 
-from PyQt5.QtGui import QDoubleValidator, QIntValidator
-from PyQt5.QtCore import Qt
+from PyQt5.QtGui import QDoubleValidator, QRegExpValidator
+from PyQt5.QtCore import Qt, QSettings, QRegExp
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
 
 import pandas as pd
 
-
-from pyboat.ui.util import MessageWindow, PandasModel, posfloatV, posintV
+from pyboat.ui.util import (
+    PandasModel,
+    posfloatV,
+    posintV,
+    default_par_dict
+)
 from pyboat.ui.analysis import mkTimeSeriesCanvas, FourierAnalyzer, WaveletAnalyzer
 from pyboat.ui.batch_process import BatchProcessWindow
 
 import pyboat
 from pyboat import plotting as pl
 
-# --- monkey patch label sizes to look good on ui ---
+# --- monkey patch label sizes to better fit the ui ---
 pl.tick_label_size = 12
 pl.label_size = 14
 
 
-class DataViewer(QWidget):
+class DataViewer(QMainWindow):
     
-    def __init__(self, data, debug=False):
+    def __init__(self, data, pos_offset, debug=False):
         super().__init__()
 
         # this is the data table
         self.df = data
 
-        self.anaWindows = {}
+        self.anaWindows = {} # allows for multiple open analysis windows
         self.w_position = 0  # analysis window position offset
 
         self.debug = debug
@@ -70,15 +69,19 @@ class DataViewer(QWidget):
         self.periodV = QDoubleValidator(bottom=1e-16, top=1e16)
         self.envelopeV = QDoubleValidator(bottom=3, top=9999999)
 
-        self.initUI()
+        self.initUI(pos_offset)
 
     # ===========    UI    ================================
 
-    def initUI(self):
+    def initUI(self, pos_offset):
 
         self.setWindowTitle(f"DataViewer - {self.df.name}")
-        self.setGeometry(80, 300, 900, 650)
+        self.setGeometry(80 + pos_offset, 300 + pos_offset, 900, 650)
 
+        # for the status bar
+        main_widget = QWidget()
+        self.statusBar()
+                
         self.tsCanvas = mkTimeSeriesCanvas()
         main_frame = QWidget()
         self.tsCanvas.setParent(main_frame)
@@ -102,22 +105,22 @@ class DataViewer(QWidget):
 
         # the signal selection box
         SignalBox = QComboBox(self)
-        SignalBox.setToolTip("..or just click on a signal in the table!")
+        SignalBox.setStatusTip("..or just click directly on a signal in the table!")
 
         main_layout_v = QVBoxLayout()  # The whole Layout
         # Data selction drop-down
         dataLabel = QLabel("Select Signal", self)
 
         dt_label = QLabel("Sampling Interval:")
-        dt_edit = QLineEdit()
-        dt_edit.setToolTip("How much time in between two recordings?")
-        dt_edit.setMinimumSize(70, 0)  # no effect :/
-        dt_edit.setValidator(posfloatV)
+        self.dt_edit = QLineEdit()
+        self.dt_edit.setStatusTip("How much time in between two recordings?")
+        self.dt_edit.setMinimumSize(70, 0)  # no effect :/
+        self.dt_edit.setValidator(posfloatV)
 
         unit_label = QLabel("Time Unit:")
-        unit_edit = QLineEdit(self)
-        unit_edit.setToolTip("Changes only the axis labels!")
-        unit_edit.setMinimumSize(70, 0)
+        self.unit_edit = QLineEdit(self)
+        self.unit_edit.setStatusTip("Changes only the axis labels..")
+        self.unit_edit.setMinimumSize(70, 0)
 
         top_bar_box = QWidget()
         top_bar_layout = QHBoxLayout()
@@ -126,10 +129,10 @@ class DataViewer(QWidget):
         top_bar_layout.addWidget(SignalBox)
         top_bar_layout.addStretch(0)
         top_bar_layout.addWidget(dt_label)
-        top_bar_layout.addWidget(dt_edit)
+        top_bar_layout.addWidget(self.dt_edit)
         top_bar_layout.addStretch(0)
         top_bar_layout.addWidget(unit_label)
-        top_bar_layout.addWidget(unit_edit)
+        top_bar_layout.addWidget(self.unit_edit)
         top_bar_layout.addStretch(0)
         top_bar_box.setLayout(top_bar_layout)
 
@@ -145,7 +148,7 @@ class DataViewer(QWidget):
         self.T_c_edit = QLineEdit()
         self.T_c_edit.setMaximumWidth(70)
         self.T_c_edit.setValidator(posfloatV)
-        self.T_c_edit.setToolTip("..in time units, e.g. 120min")
+        self.T_c_edit.setStatusTip("..in time units, e.g. 120min")
 
         sinc_options_box = QGroupBox("Sinc Detrending")
         sinc_options_layout = QGridLayout()
@@ -154,15 +157,15 @@ class DataViewer(QWidget):
         sinc_options_box.setLayout(sinc_options_layout)
 
         ## Amplitude envelope parameter
-        self.L_edit = QLineEdit()
-        self.L_edit.setMaximumWidth(70)
-        self.L_edit.setValidator(self.envelopeV)
-        self.L_edit.setToolTip("..in time units, e.g. 120min")
+        self.wsize_edit = QLineEdit()
+        self.wsize_edit.setMaximumWidth(70)
+        self.wsize_edit.setValidator(self.envelopeV)
+        self.wsize_edit.setStatusTip("..in time units, e.g. 120min")
 
         envelope_options_box = QGroupBox("Amplitude Envelope")
         envelope_options_layout = QGridLayout()
         envelope_options_layout.addWidget(QLabel("Window Size:"), 0, 0)
-        envelope_options_layout.addWidget(self.L_edit, 0, 1)
+        envelope_options_layout.addWidget(self.wsize_edit, 0, 1)
         envelope_options_box.setLayout(envelope_options_layout)
 
         # plot options box
@@ -170,16 +173,24 @@ class DataViewer(QWidget):
         plot_options_layout = QGridLayout()
 
         self.cb_raw = QCheckBox("Raw Signal", self)
+        self.cb_raw.setStatusTip("Plots the raw unfiltered signal")
+        
         self.cb_trend = QCheckBox("Trend", self)
+        self.cb_trend.setStatusTip("Plots the sinc filtered signal")
+        
         self.cb_detrend = QCheckBox("Detrended Signal", self)
+        self.cb_detrend.setStatusTip("Plots the signal after trend subtraction (detrending)")
+        
         self.cb_envelope = QCheckBox("Envelope", self)
+        self.cb_envelope.setStatusTip("Plots the estimated amplitude envelope")
 
         plotButton = QPushButton("Refresh Plot", self)
+        plotButton.setStatusTip("Updates the plot with the new filter parameter values")    
         plotButton.clicked.connect(self.doPlot)
 
         saveButton = QPushButton("Save Filter Results", self)
         saveButton.clicked.connect(self.save_out_trend)
-        saveButton.setToolTip("Writes the trend and the detrended signal into a file")
+        saveButton.setStatusTip("Writes the trend and the detrended signal into a file")
 
         ## checkbox layout
         plot_options_layout.addWidget(self.cb_raw, 0, 0)
@@ -222,39 +233,37 @@ class DataViewer(QWidget):
         tab1.parameter_box = QFormLayout()
 
         ## for wavlet params, button, etc.
-        self.T_min = QLineEdit()
-        self.T_min.setToolTip("This is the lower period limit")
-        self.step_num = QLineEdit()
-        self.step_num.insert("200")
-        self.step_num.setToolTip("Increase this number for more resolution")
-        self.T_max = QLineEdit()
-        self.T_max.setToolTip("This is the upper period limit")
+        self.Tmin_edit = QLineEdit()
+        self.Tmin_edit.setStatusTip("This is the lower period limit")
+        self.nT_edit = QLineEdit()
+        self.nT_edit.setValidator(QRegExpValidator(QRegExp('[0-9]+')))        
+        self.nT_edit.setStatusTip("Increase this number for more spectral resolution")
+        self.Tmax_edit = QLineEdit()
+        self.Tmax_edit.setStatusTip("This is the upper period limit")
 
-        self.p_max = QLineEdit()
-        self.p_max.setToolTip(
-            "Enter a fixed value for all signals or leave blank for automatic scaling"
+        self.pow_max_edit = QLineEdit()
+        self.pow_max_edit.setStatusTip(
+            "Enter a fixed value or leave blank for automatic wavelet power scaling"
         )
 
-        # self.p_max.insert(str(20)) # leave blank
-
-        T_min_lab = QLabel("Smallest period")
+        Tmin_lab = QLabel("Lowest period")
         step_lab = QLabel("Number of periods")
-        T_max_lab = QLabel("Highest  period")
-        p_max_lab = QLabel("Expected maximal power")
+        Tmax_lab = QLabel("Highest  period")
+        pow_max_lab = QLabel("Expected maximal power")
 
-        T_min_lab.setWordWrap(True)
+        Tmin_lab.setWordWrap(True)
         step_lab.setWordWrap(True)
-        T_max_lab.setWordWrap(True)
-        p_max_lab.setWordWrap(True)
+        Tmax_lab.setWordWrap(True)
+        pow_max_lab.setWordWrap(True)
 
         wletButton = QPushButton("Analyze Signal", self)
         wletButton.setStyleSheet("background-color: lightblue")
-        wletButton.setToolTip("Start the wavelet analysis!")
+        wletButton.setStatusTip("Opens the wavelet analysis..")
         wletButton.clicked.connect(self.run_wavelet_ana)
 
         batchButton = QPushButton("Analyze All..", self)
         batchButton.clicked.connect(self.run_batch)
-        batchButton.setToolTip("Sets up the analysis for all signals in the table")
+        batchButton.setStatusTip("Starts the batch processing")
 
         ## add  button to layout
         wlet_button_layout_h = QHBoxLayout()
@@ -275,10 +284,10 @@ class DataViewer(QWidget):
 
         ## Add Wavelet analyzer options to tab1.parameter_box layout
 
-        tab1.parameter_box.addRow(T_min_lab, self.T_min)
-        tab1.parameter_box.addRow(step_lab, self.step_num)
-        tab1.parameter_box.addRow(T_max_lab, self.T_max)
-        tab1.parameter_box.addRow(p_max_lab, self.p_max)
+        tab1.parameter_box.addRow(Tmin_lab, self.Tmin_edit)
+        tab1.parameter_box.addRow(Tmax_lab, self.Tmax_edit)
+        tab1.parameter_box.addRow(step_lab, self.nT_edit)        
+        tab1.parameter_box.addRow(pow_max_lab, self.pow_max_edit)
         tab1.parameter_box.addRow(self.cb_use_detrended)
         tab1.parameter_box.addRow(self.cb_use_envelope)
         tab1.parameter_box.addRow(wlet_button_layout_h)
@@ -306,8 +315,8 @@ class DataViewer(QWidget):
 
         ## Create second tab
         tab2.parameter_box = QFormLayout()
-        # tab2.parameter_box.addRow(T_min_lab,self.T_min)
-        # tab2.parameter_box.addRow(T_max_lab,self.T_max)
+        # tab2.parameter_box.addRow(Tmin_lab,self.Tmin)
+        # tab2.parameter_box.addRow(Tmax_lab,self.Tmax)
         tab2.parameter_box.addRow(self.cb_use_detrended2)
         tab2.parameter_box.addRow(self.cb_use_envelope2)
         tab2.parameter_box.addRow(self.cb_FourierT)
@@ -326,7 +335,7 @@ class DataViewer(QWidget):
         # size_pol= QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
         # ana_widget.setSizePolicy(size_pol)
 
-        # ==========Plot and Options Layout=======================================
+        # ==========Plot and Options Layout=================================
         # Merge Plotting canvas and options
         plot_and_options = QWidget()
         layout = QGridLayout()
@@ -356,16 +365,20 @@ class DataViewer(QWidget):
 
         # connect to plotting machinery
         SignalBox.activated[str].connect(self.select_signal_and_Plot)
-        self.SignalBox = SignalBox  # to modify current index by table selections
+        # to modify current index by table selections        
+        self.SignalBox = SignalBox  
 
-        # initialize parameter fields
-        dt_edit.textChanged[str].connect(self.qset_dt)
-        dt_edit.insert(str(1))  # initial sampling interval is 1
+        # --- connect some parameter fields ---
+        
+        self.dt_edit.textChanged[str].connect(self.qset_dt)
+        self.unit_edit.textChanged[str].connect(self.qset_time_unit)
+        
+        # --- initialize parameter fields from settings ---
 
-        unit_edit.textChanged[str].connect(self.qset_time_unit)
-        unit_edit.insert("min")  # standard time unit is minutes
-
-        self.setLayout(main_layout_v)
+        self.load_settings()
+                
+        main_widget.setLayout(main_layout_v)
+        self.setCentralWidget(main_widget)        
         self.show()
 
         # trigger initial plot?!
@@ -432,17 +445,16 @@ class DataViewer(QWidget):
             self.doPlot()
 
     def toggle_envelope(self, state):
-
-        # trying to plot the trend
-        if state == Qt.Checked:                
-            L = self.get_L(self.L_edit)
-            if not L:
-                self.cb_envelope.setChecked(False)
-                self.cb_use_envelope.setChecked(False)
-                 
-         
+                         
         # signal selected?
         if self.signal_id:
+            # trying to plot the envelope
+            if state == Qt.Checked:                
+                window_size = self.get_wsize(self.wsize_edit)
+                if not window_size:
+                    self.cb_envelope.setChecked(False)
+                    self.cb_use_envelope.setChecked(False)
+            
             self.doPlot()
 
     # connected to unit_edit
@@ -464,8 +476,8 @@ class DataViewer(QWidget):
         t = text.replace(",", ".")
         try:
             self.dt = float(t)
-            self.set_initial_periods(force=True)
-            self.set_initial_T_c(force=True)
+            self.set_initial_periods(force=False)
+            self.set_initial_T_c(force=False)
             # update  Validators
             self.periodV = QDoubleValidator(bottom=2 * self.dt, top=1e16)
             self.envelopeV = QDoubleValidator(
@@ -485,7 +497,6 @@ class DataViewer(QWidget):
         if self.debug:
             print("dt set to:", self.dt)
 
-
     def get_T_c(self, T_c_edit):
 
         '''
@@ -505,36 +516,63 @@ class DataViewer(QWidget):
         
         # empty line edit
         except ValueError:
-            self.NoTrend = MessageWindow(
-                "Detrending parameter not set,\n" + "specify a cut-off period!",
-                "No Trend",
-            )
-            
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle('Missing Parameter')
+            msgBox.setText(
+                'Detrending parameter not set, specify a cut-off period!')
+            msgBox.exec()
+                        
             if self.debug:
                 print("T_c ValueError", tc)
             return None
         
-    def get_L(self, L_edit):
+    def get_wsize(self, wsize_edit):
 
         # value checking done by validator, accepts also comma '1,1' !
-        L = L_edit.text().replace(",", ".")
+        window_size = wsize_edit.text().replace(",", ".")
         try:
-            L = int(L)
+            window_size = float(window_size)
 
         # empty line edit
         except ValueError:
-            self.NoTrend = MessageWindow(
-                "Envelope parameter not set,\n" + "specify a sliding window size!",
-                "No Envelope",
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle('Missing Parameter')            
+            msgBox.setText(
+                'Amplitude envelope parameter not set, specify a sliding window size!')
+
+            msgBox.exec()
             
             if self.debug:
-                print("L ValueError", L)
+                print("window_size ValueError", window_size)
+            return None
+
+        if window_size / self.dt < 4:
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle("Out of Bounds")
+            msgBox.setText(
+                f'''Minimal sliding window size for envelope estimation is {4*self.dt} {self.time_unit}!''')
+            msgBox.exec()
+
+            return None
+
+        if window_size / self.dt > self.df.shape[0]:
+            max_window_size = self.df.shape[0] * self.dt
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle("Out of Bounds")
+            msgBox.setText(
+                f"Maximal sliding window size for envelope estimation is {max_window_size:.2f} {self.time_unit}!")
+            msgBox.exec()
+            
             return None
         
         if self.debug:
-            print("L set to:", L)
-        return L
+            print("window_size set to:", window_size)
+            
+        return window_size
 
     def set_initial_periods(self, force=False):
 
@@ -545,21 +583,21 @@ class DataViewer(QWidget):
         if self.debug:
             print("set_initial_periods called")
 
-        # check if a T_min was already entered
+        # check if a Tmin was already entered
         # or rewrite if enforced
-        if not bool(self.T_min.text()) or force:
-            self.T_min.clear()
-            self.T_min.insert(str(2 * self.dt))  # Nyquist
+        if not bool(self.Tmin_edit.text()) or force:
+            self.Tmin_edit.clear()
+            self.Tmin_edit.insert(str(2 * self.dt))  # Nyquist
 
         if np.any(self.raw_signal):  # check if raw_signal already selected
-            # check if a T_max was already entered
-            if not bool(self.T_max.text()) or force:
+            # check if a Tmax was already entered
+            if not bool(self.Tmax_edit.text()) or force:
                 # default is 1/4 the observation time
-                self.T_max.clear()
-                T_max_ini = self.dt * 1 / 4 * len(self.raw_signal)
+                self.Tmax_edit.clear()
+                Tmax_ini = self.dt * 1 / 4 * len(self.raw_signal)
                 if self.dt > 0.1:
-                    T_max_ini = int(T_max_ini)
-                self.T_max.insert(str(T_max_ini))
+                    Tmax_ini = int(Tmax_ini)
+                self.Tmax_edit.insert(str(Tmax_ini))
 
     def set_initial_T_c(self, force=False):
         if self.debug:
@@ -570,7 +608,7 @@ class DataViewer(QWidget):
             if (
                 not bool(self.T_c_edit.text()) or force
             ):  
-                # default is 1.5 * T_max -> 3/8 the observation time
+                # default is 1.5 * Tmax -> 3/8 the observation time
                 self.T_c_edit.clear()
                 
                 T_c_ini = self.dt * 3 / 8 * len(self.raw_signal)
@@ -581,7 +619,6 @@ class DataViewer(QWidget):
 
                 self.T_c_edit.insert(str(T_c_ini))
 
-
     def set_wlet_pars(self):
 
         '''
@@ -589,25 +626,24 @@ class DataViewer(QWidget):
         of the 'Analysis' input box reading the following
         QLineEdits:
 
-        self.Tmin
-        self.Tmax
-        self.step_num
-        self.pmax
+        self.Tmin_edit
+        self.Tmax_edit
+        self.nT_edit
+        self.pow_max_edit
 
         Further the checkboxes regarding detrending and amplitude
         normalization are evaluated. And
 
-        self.get_L()
+        self.get_wsize()
         self.get_T_c()
 
-        are called if needed. These respective parameters are set to False
-        if the opereation in question is not requested.
+        are called if needed. 
 
         Returns
         -------
 
         wlet_pars : dictionary holding the retrieved parameters,
-                    L and T_c are set to None if no amplitude
+                    window_size and T_c are set to None if no amplitude
                     normalization or detrending operation should be done
 
         '''
@@ -619,27 +655,45 @@ class DataViewer(QWidget):
 
         # -- read all the QLineEdits --
 
-        text = self.T_min.text()
-        T_min = text.replace(",", ".")
-        check, _, _ = vali.validate(T_min, 0)
+        text = self.Tmin_edit.text()
+        text = text.replace(",", ".")
+        check, _, _ = vali.validate(text, 0)
         if self.debug:
-            print("Min periodValidator output:", check, "value:", T_min)
+            print("Min periodValidator output:", check, "value:", text)
         if check == 0:
-            self.OutOfBounds = MessageWindow(
-                "Wavelet periods out of bounds!", "Value Error"
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setText("Lowest period out of bounds!")
+            msgBox.exec()
+
             return False
 
-        wlet_pars['T_min'] = float(T_min)
+        Tmin = float(text)
+
+        if Tmin < 2 * self.dt:
+
+            Tmin = 2 * self.dt
+            self.Tmin_edit.clear()
+            self.Tmin_edit.insert(str(Tmin))
+            
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle('Warning')
+            msg = f"Lowest period set to Nyquist limit: {Tmin} {self.time_unit}!"
+            msgBox.setText(msg)
+            msgBox.exec()
+        
+        wlet_pars['Tmin'] = Tmin
                 
-        step_num = self.step_num.text()
+        step_num = self.nT_edit.text()
         check, _, _ = posintV.validate(step_num, 0)
         if self.debug:
             print("# Periods posintValidator:", check, "value:", step_num)
         if check == 0:
-            self.OutOfBounds = MessageWindow(
-                "The Number of periods must be a positive integer!", "Value Error"
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setText(
+                "The Number of periods must be a positive integer!")
+            msgBox.exec()
             return False
 
         wlet_pars['step_num'] = int(step_num)
@@ -656,31 +710,37 @@ class DataViewer(QWidget):
             else:
                 return False        
         
-        text = self.T_max.text()
-        T_max = text.replace(",", ".")
-        check, _, _ = vali.validate(T_max, 0)
+        text = self.Tmax_edit.text()
+        Tmax = text.replace(",", ".")
+        check, _, _ = vali.validate(Tmax, 0)
         if self.debug:
             print("Max periodValidator output:", check)
-            print(f"Max period value: {self.T_max.text()}")
+            print(f"Max period value: {self.Tmax_edit.text()}")
         if check == 0 or check == 1:
-            self.OutOfBounds = MessageWindow(
-                "Wavelet highest period out of bounds!", "Value Error"
-            )
-            return False
-        wlet_pars['T_max'] = float(T_max)
 
-        text = self.p_max.text()
-        p_max = text.replace(",", ".")
-        check, _, _ = posfloatV.validate(p_max, 0)  # checks for positive float
+            msgBox = QMessageBox()
+            msgBox.setText("Highest periods out of bounds!")
+            msgBox.exec()
+            
+            return False
+        wlet_pars['Tmax'] = float(Tmax)
+
+        text = self.pow_max_edit.text()
+        pow_max = text.replace(",", ".")
+        check, _, _ = posfloatV.validate(pow_max, 0)  # checks for positive float
         if check == 0:
-            self.OutOfBounds = MessageWindow("Powers are positive!", "Value Error")
+
+            msgBox = QMessageBox()
+            msgBox.setText("Maximal power must be positive!")
+            msgBox.exec()
+
             return False
 
         # check for empty string:
-        if p_max:
-            wlet_pars['p_max'] = float(p_max)
+        if pow_max:
+            wlet_pars['pow_max'] = float(pow_max)
         else:
-            wlet_pars['p_max'] = None
+            wlet_pars['pow_max'] = None
 
         # -- the checkboxes --
             
@@ -692,17 +752,17 @@ class DataViewer(QWidget):
             wlet_pars['T_c'] = T_c
         else:
             # indicates no detrending requested
-            wlet_pars['T_c'] = False
+            wlet_pars['T_c'] = None
             
         # amplitude normalization is downstram of detrending!
         if self.cb_use_envelope.isChecked():
-            L = self.get_L(self.L_edit)        
-            if L is None:
+            window_size = self.get_wsize(self.wsize_edit)        
+            if window_size is None:
                 return False # abort settings                        
-            wlet_pars['L'] = L
+            wlet_pars['window_size'] = window_size
         else:
             # indicates no ampl. normalization
-            wlet_pars['L'] = False
+            wlet_pars['window_size'] = None
                     
         # success!
         return wlet_pars
@@ -722,28 +782,31 @@ class DataViewer(QWidget):
             NaNswitches = np.sum( np.diff( np.isnan(raw_signal) ) )
             if NaNswitches > 1:
                 print(f'Warning, non-contiguous NaN region found in {signal_id}!')
-                
-                self.NonContiguous = MessageWindow(
-                '''
-                Non contiguous regions of missing values 
-                encountered, using linear interpolation. 
 
-                Try 'Import..' from the main menu 
-                to interpolate missing values in all signals!
-                '''
-                    , "Missing Values")
+                msgBox = QMessageBox()
+                msgBox.setText(
+                    '''
+                    Non contiguous regions of missing values 
+                    encountered, using linear interpolation. 
+
+                    Try 'Import..' from the main menu 
+                    to interpolate missing values for all signals!
+                    ''')                
+                msgBox.exec()
+                
                 self.raw_signal = pyboat.core.interpolate_NaNs(raw_signal)
             else:
                 # remove contiguous (like trailing) NaN region
                 self.raw_signal = raw_signal[~np.isnan(raw_signal)]
-            
+
             self.tvec = np.arange(0, len(self.raw_signal), step=1) * self.dt
             return True  # success
 
         else:
-            self.NoSignalSelected = MessageWindow(
-                "Please select a signal!", "No Signal"
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setText("Please select a signal!")
+            msgBox.exec()
             return False
 
     def calc_trend(self):
@@ -761,31 +824,15 @@ class DataViewer(QWidget):
 
     def calc_envelope(self):
 
-        L = self.get_L(self.L_edit)
-        if not L:
+        window_size = self.get_wsize(self.wsize_edit)
+        if not window_size:
             return
         if self.debug:
-            print("Calculating envelope with L = ", L)
+            print("Calculating envelope with window_size = ", window_size)
             
-        if L / self.dt < 4:
-            self.OutOfBounds = MessageWindow(
-                f"Minimum sliding\nwindow size is {4*self.dt}{self.time_unit} !",
-                "Value Error",
-            )
-            L = None
-            return
-
-        if L / self.dt > self.df.shape[0]:
-            maxL = self.df.shape[0] * self.dt
-            self.OutOfBounds = MessageWindow(
-                f"Maximum sliding window\nsize is {maxL:.2f} {self.time_unit}!",
-                "Value Error",
-            )
-            L = None
-            return
 
         # cut of frequency set?!
-        if self.cb_trend.isChecked():
+        if self.get_T_c(self.T_c_edit):
 
             trend = self.calc_trend()
             if trend is None:
@@ -793,7 +840,7 @@ class DataViewer(QWidget):
             
             signal = self.raw_signal - trend
             envelope = pyboat.sliding_window_amplitude(signal,
-                                                       window_size = L,
+                                                       window_size,
                                                        dt = self.dt)
 
             if self.cb_detrend.isChecked():
@@ -806,11 +853,11 @@ class DataViewer(QWidget):
         # otherwise add the mean
         else:
             if self.debug:
-                print("calculating envelope for raw signal", L)
+                print("calculating envelope for raw signal", window_size)
 
             mean = self.raw_signal.mean()
             envelope = pyboat.sliding_window_amplitude(
-                self.raw_signal, window_size = L, dt = self.dt
+                self.raw_signal, window_size = window_size, dt = self.dt
             )
             return envelope + mean
 
@@ -872,7 +919,7 @@ class DataViewer(QWidget):
             ax2 = pl.draw_detrended(
                 ax1, time_vector=self.tvec, detrended=self.raw_signal - trend
             )
-            ax2.legend(fontsize=pl.tick_label_size)
+            ax2.legend(fontsize=pl.tick_label_size, loc='lower left')
         if envelope is not None and not self.cb_detrend.isChecked():
             pl.draw_envelope(ax1, time_vector=self.tvec, envelope=envelope)
 
@@ -893,9 +940,12 @@ class DataViewer(QWidget):
         """ run the Wavelet Analysis """
 
         if not np.any(self.raw_signal):
-            self.NoSignalSelected = MessageWindow(
-                "Please select a signal first!", "No Signal"
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle("No Signal")
+            msgBox.setText("Please select a signal first!")
+            msgBox.exec()
+
             return False
 
         wlet_pars = self.set_wlet_pars()  # Error handling done there
@@ -910,20 +960,18 @@ class DataViewer(QWidget):
         else:
             signal = self.raw_signal
 
-
         if self.cb_use_envelope.isChecked():
-            L = self.get_L(self.L_edit)
-            signal = pyboat.normalize_with_envelope(signal, L, dt = self.dt)
+            window_size = self.get_wsize(self.wsize_edit)
+            signal = pyboat.normalize_with_envelope(signal, window_size, dt = self.dt)
 
         self.w_position += 20
-
         
         self.anaWindows[self.w_position] = WaveletAnalyzer(
             signal=signal,
             dt=self.dt,
-            T_min = wlet_pars['T_min'],
-            T_max = wlet_pars['T_max'],
-            p_max = wlet_pars['p_max'],
+            Tmin = wlet_pars['Tmin'],
+            Tmax = wlet_pars['Tmax'],
+            pow_max = wlet_pars['pow_max'],
             step_num=wlet_pars['step_num'],
             position=self.w_position,
             signal_id=self.signal_id,
@@ -953,30 +1001,28 @@ class DataViewer(QWidget):
 
         return
         print("batch processing done!")
-        msg = f"Processed {Nproc} signals!\n ..saved results to {dir_name}"
-        self.msg = MessageWindow(msg, "Finished")
-
+        
     def run_fourier_ana(self):
         if not np.any(self.raw_signal):
-            self.NoSignalSelected = MessageWindow(
-                "Please select a signal first!", "No Signal"
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle("No Signal")
+            msgBox.setText("Please select a signal first!")
+            msgBox.exec()
             return False
 
         # shift new analyser windows
         self.w_position += 20
 
-
         if self.cb_use_detrended2.isChecked():
             trend = self.calc_trend()
             signal = self.raw_signal - trend
         else:
             signal = self.raw_signal
 
-
         if self.cb_use_envelope2.isChecked():
-            L = self.get_L(self.L_edit)
-            signal = pyboat.normalize_with_envelope(signal, L, self.dt)
+            window_size = self.get_wsize(self.wsize_edit)
+            signal = pyboat.normalize_with_envelope(signal, window_size, self.dt)
 
         # periods or frequencies?
         if self.cb_FourierT.isChecked():
@@ -996,9 +1042,11 @@ class DataViewer(QWidget):
     def save_out_trend(self):
 
         if not np.any(self.raw_signal):
-            self.NoSignalSelected = MessageWindow(
-                "Please select a signal first!", "No Signal"
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle("No Signal")
+            msgBox.setText("Please select a signal first!")
+            msgBox.exec()
             return
 
         if self.debug:
@@ -1020,14 +1068,18 @@ class DataViewer(QWidget):
         dialog = QFileDialog()
         options = QFileDialog.Options()
 
+        settings = QSettings()        
         # ----------------------------------------------------------
-        default_name = "trend_" + str(self.signal_id)
-        format_filter = "Text File (*.txt);; CSV ( *.csv);; Excel (*.xlsx)"
+        base_name = str(self.signal_id).replace(' ', '-')
+        dir_path = settings.value('dir_name', os.path.curdir)
+
+        default_name = os.path.join(dir_path,
+                                    base_name + '_trend')
+        format_filter = "Text File (*.txt);; csv ( *.csv);; MS Excel (*.xlsx)"
         # -----------------------------------------------------------
         file_name, sel_filter = dialog.getSaveFileName(
-            self, "Save as", default_name, format_filter, None, options=options
+            self, "Save as", directory=default_name, filter=format_filter, options=options
         )
-
         # dialog cancelled
         if not file_name:
             return
@@ -1040,17 +1092,18 @@ class DataViewer(QWidget):
             print("extracted extension:", file_ext)
 
         if file_ext not in ["txt", "csv", "xlsx"]:
-            self.e = MessageWindow(
-                "Ouput format not supported..\n"
-                + "Please append .txt, .csv or .xlsx\n"
-                + "to the file name!",
-                "Unknown format",
-            )
+
+            msgBox = QMessageBox()
+            msgBox.setWindowTitle("Unknown File Format")
+            msgBox.setText(
+                "Please append .txt, .csv or .xlsx to the file name!")
+            msgBox.exec()            
             return
 
         # ------the write out calls to pandas----------------
 
-        float_format = "%.2f"  # still old style :/
+        # defaults to 3 decimals
+        float_format = settings.value('float_format', '%.3f')
 
         if file_ext == "txt":
             df_out.to_csv(file_name, index=False, sep="\t", float_format=float_format)
@@ -1067,3 +1120,30 @@ class DataViewer(QWidget):
             return
         if self.debug:
             print("Saved!")
+
+    def load_settings(self):
+        
+        settings = QSettings()
+
+        # map parameter keys to edits
+        key_to_edit = {
+            'dt' : self.dt_edit,
+            'time_unit' : self.unit_edit,
+            'cut_off' : self.T_c_edit,
+            'window_size' : self.wsize_edit,
+            'Tmin' : self.Tmin_edit,
+            'Tmax' : self.Tmax_edit,
+            'nT' : self.nT_edit,
+            'pow_max' : self.pow_max_edit,
+            'float_format' : None,
+            'graphics_format' : None
+        }
+
+        # load defaults from dict or restore values
+        for key, value in default_par_dict.items():
+            val = settings.value(key, value)
+            edit = key_to_edit[key]
+            # some fields might be left empty for dynamic defaults
+            if edit and (val is not None):
+                edit.clear() # to be on the safe side
+                edit.insert(str(val))
