@@ -1,19 +1,13 @@
 from PyQt5.QtWidgets import (
     QCheckBox,
-    QComboBox,
-    QFileDialog,
-    QAction,
     QMainWindow,
-    QApplication,
     QLabel,
     QLineEdit,
     QPushButton,
     QMessageBox,
-    QSizePolicy,
     QWidget,
     QVBoxLayout,
     QHBoxLayout,
-    QDialog,
     QGroupBox,
     QFormLayout,
     QGridLayout,
@@ -21,7 +15,7 @@ from PyQt5.QtWidgets import (
     QSpacerItem,
 )
 
-from PyQt5.QtGui import QDoubleValidator, QIntValidator, QScreen
+from PyQt5.QtGui import QDoubleValidator, QIntValidator
 from PyQt5.QtCore import Qt
 
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
@@ -38,7 +32,7 @@ pl.tick_label_size = 12
 pl.label_size = 14
 
 
-class SynthSignalGen(QWidget):
+class SynthSignalGen(QMainWindow):
 
     """
     This is basically a clone of the
@@ -57,10 +51,8 @@ class SynthSignalGen(QWidget):
         self.debug = debug
 
         self.raw_signal = None
-        self.Nt = 500  # needs to be set here..
+        self.Nt = 500  # needs to be set here for initial plot..
         self.dt = 1  # .. to create the default signal
-        self.T_c = None  # gets initialized from the UI -> qset_T_c
-        self.L = None  # gets initialized from the UI -> qset_L
         self.tvec = None  # gets initialized by vector_prep
         self.time_unit = None  # gets initialized by qset_time_unit
 
@@ -77,6 +69,9 @@ class SynthSignalGen(QWidget):
         self.setWindowTitle(f"Synthetic Signal Generator")
         self.setGeometry(80, 300, 900, 650)
 
+        main_widget = QWidget()
+        self.statusBar()
+
         self.tsCanvas = mkTimeSeriesCanvas()
         main_frame = QWidget()
         self.tsCanvas.setParent(main_frame)
@@ -103,11 +98,11 @@ class SynthSignalGen(QWidget):
 
         Nt_label = QLabel("# Samples")
         self.Nt_edit = QLineEdit()
-        self.Nt_edit.setToolTip(
-            "Number of data points, minimum is 10, maximum is 10 000!"
+        self.Nt_edit.setStatusTip(
+            "Number of data points, minimum is 10, maximum is 25 000!"
         )
         set_max_width(self.Nt_edit, iwidth)
-        self.Nt_edit.setValidator(QIntValidator(bottom=10, top=10000))
+        self.Nt_edit.setValidator(QIntValidator(bottom=10, top=25000))
 
 
         # --- the basic settings box ---
@@ -116,33 +111,34 @@ class SynthSignalGen(QWidget):
         basics_box_layout.setSpacing(2.5)
         basics_box.setLayout(basics_box_layout)
 
+        basics_box_layout.addWidget(Nt_label)
+        basics_box_layout.addWidget(self.Nt_edit)        
         basics_box_layout.addWidget(dt_label)
         basics_box_layout.addWidget(dt_edit)
-        basics_box_layout.addWidget(Nt_label)
-        basics_box_layout.addWidget(self.Nt_edit)
         basics_box_layout.addWidget(unit_label)
         basics_box_layout.addWidget(unit_edit)
+        
         basics_box_layout.addStretch(0)
 
         # --- chirp 1 ---
 
         T11_label = QLabel("Initial Period")
         self.T11_edit = QLineEdit()
-        self.T11_edit.setToolTip("Period at the beginning of the signal")
+        self.T11_edit.setStatusTip("Period at the beginning of the signal")
         set_max_width(self.T11_edit, iwidth)
         self.T11_edit.setValidator(posfloatV)
         self.T11_edit.insert(str(50))  # initial period of chirp 1
 
         T12_label = QLabel("Final Period")
         self.T12_edit = QLineEdit()
-        self.T12_edit.setToolTip("Period at the end of the signal")
+        self.T12_edit.setStatusTip("Period at the end of the signal")
         set_max_width(self.T12_edit, iwidth)
         self.T12_edit.setValidator(posfloatV)
         self.T12_edit.insert(str(150))  # initial period of chirp 1
 
         A1_label = QLabel("Amplitude")
         self.A1_edit = QLineEdit()
-        self.A1_edit.setToolTip("The amplitude :)")
+        self.A1_edit.setStatusTip("The amplitude :)")
         set_max_width(self.A1_edit, iwidth)
         self.A1_edit.setValidator(posfloatV)
         self.A1_edit.insert(str(1))  # initial amplitude
@@ -167,21 +163,21 @@ class SynthSignalGen(QWidget):
 
         T21_label = QLabel("Initial Period")
         self.T21_edit = QLineEdit()
-        self.T21_edit.setToolTip("Period at the beginning of the signal")
+        self.T21_edit.setStatusTip("Period at the beginning of the signal")
         set_max_width(self.T21_edit, iwidth)
         self.T21_edit.setValidator(posfloatV)
         self.T21_edit.insert(str(1000))  # initial period of chirp 1
 
         T22_label = QLabel("Final Period")
         self.T22_edit = QLineEdit()
-        self.T22_edit.setToolTip("Period at the end of the signal")
+        self.T22_edit.setStatusTip("Period at the end of the signal")
         set_max_width(self.T22_edit, iwidth)
         self.T22_edit.setValidator(posfloatV)
         self.T22_edit.insert(str(1000))  # initial period of chirp 1
 
         A2_label = QLabel("Amplitude")
         self.A2_edit = QLineEdit()
-        self.A2_edit.setToolTip("Can be negative to induce different trends..")
+        self.A2_edit.setStatusTip("Can be negative to induce different trends..")
         set_max_width(self.A2_edit, iwidth)
         self.A2_edit.setValidator(floatV)
         self.A2_edit.insert(str(2))  # initial amplitude
@@ -203,26 +199,27 @@ class SynthSignalGen(QWidget):
 
         # --- the AR1 box ---
         self.noise_box = QGroupBox("Noise")
+        self.noise_box.setStatusTip("Adds colored AR(1) noise to the signal")
         self.noise_box.setCheckable(True)
         self.noise_box.setChecked(False)
         noise_box_layout = QVBoxLayout()
         self.noise_box.setLayout(noise_box_layout)
 
-        alpha_label = QLabel("AR1 parameter")
+        alpha_label = QLabel("AR(1) parameter")
         self.alpha_edit = QLineEdit(parent=self.noise_box)
-        self.alpha_edit.setToolTip("0 is white noise, must be smaller than 1!")
+        self.alpha_edit.setStatusTip("0 is white noise, must be smaller than 1!")
         set_max_width(self.alpha_edit, iwidth)
         # does not work as expected :/
-        V1 = QDoubleValidator()
-        V1.setNotation(QDoubleValidator.StandardNotation)
-        V1.setDecimals(200)
-        V1.setRange(0.000, 0.9999)
+        V1 = QDoubleValidator(bottom = 0, top = 0.99)
+        # V1.setNotation(QDoubleValidator.StandardNotation)
+        # V1.setDecimals(5)
+        # V1.setRange(0.0, 0.99)
         self.alpha_edit.setValidator(V1)
-        self.alpha_edit.insert("0.2")  # initial AR1
+        self.alpha_edit.insert("0.2")  # initial AR(1)
 
         d_label = QLabel("Noise Strength")
         self.d_edit = QLineEdit(parent=self.noise_box)
-        self.d_edit.setToolTip("Compare to set oscillator amplitudes..")
+        self.d_edit.setStatusTip("Relative to oscillator amplitudes..")
         set_max_width(self.d_edit, iwidth)
         self.d_edit.setValidator(QDoubleValidator(bottom=0, top=999999))
         self.d_edit.insert("0.5")  # initial noise strength
@@ -237,7 +234,7 @@ class SynthSignalGen(QWidget):
 
         tau_label = QLabel("Decay Time")
         self.tau_edit = QLineEdit()
-        self.tau_edit.setToolTip(
+        self.tau_edit.setStatusTip(
             "Time after which the signal decayed to around a third of the initial amplitude"
         )
         set_max_width(self.tau_edit, iwidth)
@@ -258,7 +255,7 @@ class SynthSignalGen(QWidget):
         # --- the create signal button
         ssgButton = QPushButton("Synthesize Signal", self)
         ssgButton.clicked.connect(self.create_signal)
-        ssgButton.setToolTip(
+        ssgButton.setStatusTip(
             "Click again with same settings for different noise realizations"
         )
         ssgButton.setStyleSheet("background-color: orange")
@@ -266,9 +263,9 @@ class SynthSignalGen(QWidget):
         ssgButton_box = QWidget()
         ssgButton_box_layout = QHBoxLayout()
         ssgButton_box.setLayout(ssgButton_box_layout)
-        ssgButton_box_layout.addStretch(0)
+        #ssgButton_box_layout.addStretch(0)
         ssgButton_box_layout.addWidget(ssgButton)
-        ssgButton_box_layout.addStretch(0)
+        #ssgButton_box_layout.addStretch(0)
 
         control_grid = QWidget()
         control_grid_layout = QGridLayout()
@@ -303,14 +300,14 @@ class SynthSignalGen(QWidget):
         sinc_options_box.setLayout(sinc_options_layout)
 
         ## Amplitude envelope parameter
-        L_edit = QLineEdit()
-        L_edit.setMaximumWidth(70)
-        L_edit.setValidator(self.envelopeV)
+        self.L_edit = QLineEdit()
+        self.L_edit.setMaximumWidth(70)
+        self.L_edit.setValidator(self.envelopeV)
 
         envelope_options_box = QGroupBox("Amplitude Envelope")
         envelope_options_layout = QGridLayout()
         envelope_options_layout.addWidget(QLabel("Window Size:"), 0, 0)
-        envelope_options_layout.addWidget(L_edit, 0, 1)
+        envelope_options_layout.addWidget(self.L_edit, 0, 1)
         envelope_options_box.setLayout(envelope_options_layout)
 
         # plot options box
@@ -367,21 +364,21 @@ class SynthSignalGen(QWidget):
 
         ## for wavlet params, button, etc.
         self.T_min = QLineEdit()
-        self.T_min.setToolTip("This is the lower period limit")
+        self.T_min.setStatusTip("This is the lower period limit")
         self.step_num = QLineEdit()
         self.step_num.insert("200")
-        self.step_num.setToolTip("Increase this number for more resolution")
+        self.step_num.setStatusTip("Increase this number for more resolution")
         self.T_max = QLineEdit()
-        self.T_max.setToolTip("This is the upper period limit")
+        self.T_max.setStatusTip("This is the upper period limit")
 
         self.p_max = QLineEdit()
-        self.p_max.setToolTip(
+        self.p_max.setStatusTip(
             "Enter a fixed value for all signals or leave blank for automatic scaling"
         )
 
         # self.p_max.insert(str(20)) # leave blank
 
-        T_min_lab = QLabel("Smallest period")
+        T_min_lab = QLabel("Lowest period")
         step_lab = QLabel("Number of periods")
         T_max_lab = QLabel("Highest  period")
         p_max_lab = QLabel("Expected maximal power")
@@ -393,7 +390,7 @@ class SynthSignalGen(QWidget):
 
         wletButton = QPushButton("Analyze Signal", self)
         wletButton.clicked.connect(self.run_wavelet_ana)
-        wletButton.setToolTip("Start the wavelet analysis!")
+        wletButton.setStatusTip("Start the wavelet analysis!")
         wletButton.setStyleSheet("background-color: lightblue")
 
         ## add  button to layout
@@ -414,8 +411,8 @@ class SynthSignalGen(QWidget):
         ## Add Wavelet analyzer options to tab1.parameter_box layout
 
         tab1.parameter_box.addRow(T_min_lab, self.T_min)
-        tab1.parameter_box.addRow(step_lab, self.step_num)
         tab1.parameter_box.addRow(T_max_lab, self.T_max)
+        tab1.parameter_box.addRow(step_lab, self.step_num)        
         tab1.parameter_box.addRow(p_max_lab, self.p_max)
         tab1.parameter_box.addRow(self.cb_use_detrended)
         tab1.parameter_box.addRow(self.cb_use_envelope)
@@ -494,15 +491,13 @@ class SynthSignalGen(QWidget):
         self.Nt_edit.insert(str(self.Nt))  # initial signal length
         dt_edit.insert(str(self.dt))  # initial sampling interval is 1
 
-        self.T_c_edit.textChanged[str].connect(self.qset_T_c)
-        L_edit.textChanged[str].connect(self.qset_L)
-
         # create the default signal
         self.create_signal()
 
-        self.setLayout(main_layout_v)
+        main_widget.setLayout(main_layout_v)
+        self.setCentralWidget(main_widget)        
         self.show()
-
+ 
     # probably all the toggle state variables are not needed -> read out checkboxes directly
     def toggle_raw(self, state):
         if state == Qt.Checked:
@@ -512,36 +507,34 @@ class SynthSignalGen(QWidget):
 
         if self.raw_signal is not None:
             self.doPlot()
-
     def toggle_trend(self, state):
 
         if self.debug:
-            print("old state:", self.cb_trend.isChecked())
+            print("new state:", self.cb_trend.isChecked())
 
+        # trying to plot the trend
         if state == Qt.Checked:
-            # user warning - no effect without T_c set
-            if not self.T_c:
-                self.NoTrend = MessageWindow(
-                    "Specify a cut-off period!", "Missing value"
-                )
-
+            T_c = self.get_T_c(self.T_c_edit)
+            if not T_c:
+                self.cb_trend.setChecked(False)
+                # self.cb_use_detrended.setChecked(False)
+                
         # signal selected?
-
-        if self.raw_signal is not None:
+        if np.any(self.raw_signal):
             self.doPlot()
 
     def toggle_envelope(self, state):
-        if state == Qt.Checked:
-
-            # user warning - no effect without L set
-            if not self.L:
-                self.NoEnvelope = MessageWindow(
-                    "Specify a sliding window size!", "Missing value"
-                )
 
+        # trying to plot the envelope
+        if state == Qt.Checked:                
+            L = self.get_L(self.L_edit)
+            if not L:
+                self.cb_envelope.setChecked(False)
+                self.cb_use_envelope.setChecked(False)
+                 
+         
         # signal selected?
-
-        if self.raw_signal is not None:
+        if np.any(self.raw_signal):
             self.doPlot()
 
     # connected to unit_edit
@@ -577,44 +570,57 @@ class SynthSignalGen(QWidget):
         if self.debug:
             print("dt set to:", self.dt)
 
-    # connected to T_c_edit
-    def qset_T_c(self, text):
+
+    def get_T_c(self, T_c_edit):
+
+        '''
+        Uses self.T_c_edit, argument just for clarity. Checks
+        for empty input, this function only gets called when
+        a detrending operation is requested. Hence, an empty
+        QLineEdit will display a user warning and return nothing..
+        '''
 
         # value checking done by validator, accepts also comma '1,1' !
-        tc = text.replace(",", ".")
+        tc = T_c_edit.text().replace(",", ".")        
         try:
-            self.T_c = float(tc)
-
+            T_c = float(tc)
+            if self.debug:
+                print("T_c set to:", T_c)
+            return T_c
+        
         # empty line edit
         except ValueError:
+            self.NoTrend = MessageWindow(
+                "Detrending parameter not set,\n" + "specify a cut-off period!",
+                "No Trend",
+            )
+            
             if self.debug:
-                print("T_c ValueError", text)
-            pass
-
-        if self.debug:
-            print("T_c set to:", self.T_c)
-
-    # connected to T_c_edit
-    def qset_L(self, text):
+                print("T_c ValueError", tc)
+            return None
+        
+    def get_L(self, L_edit):
 
         # value checking done by validator, accepts also comma '1,1' !
-        L = text.replace(",", ".")
+        L = L_edit.text().replace(",", ".")
         try:
             L = int(L)
 
         # empty line edit
         except ValueError:
+            self.NoTrend = MessageWindow(
+                "Envelope parameter not set,\n" + "specify a sliding window size!",
+                "No Envelope",
+            )
+            
             if self.debug:
-                print("L ValueError", text)
-            pass
-
-        # still in sampling interval units
-        self.L = int(L)
-
+                print("L ValueError", L)
+            return None
+        
         if self.debug:
-            print("L set to:", self.L)
-
-
+            print("L set to:", L)
+        return L
+            
     def set_initial_periods(self, force=False):
 
         """
@@ -660,13 +666,42 @@ class SynthSignalGen(QWidget):
                     T_c_ini = np.round(T_c_ini, 3)
                 self.T_c_edit.insert(str(T_c_ini))
 
-    # retrieve and check set wavelet parameters
     def set_wlet_pars(self):
 
+        '''
+        Retrieves and checks the set wavelet parameters
+        of the 'Analysis' input box reading the following
+        QLineEdits:
+
+        self.Tmin
+        self.Tmax
+        self.step_num
+        self.pmax
+
+        Further the checkboxes regarding detrending and amplitude
+        normalization are evaluated. And
+
+        self.get_L()
+        self.get_T_c()
+
+        are called if needed. These respective parameters are set to False
+        if the opereation in question is not requested.
+
+        Returns
+        -------
+
+        wlet_pars : dictionary holding the retrieved parameters,
+                    L and T_c are set to None if no amplitude
+                    normalization or detrending operation should be done
+
+        '''
+
+        wlet_pars = {}
+
         # period validator
         vali = self.periodV
 
-        # read all the LineEdits:
+        # -- read all the QLineEdits --
 
         text = self.T_min.text()
         T_min = text.replace(",", ".")
@@ -678,8 +713,9 @@ class SynthSignalGen(QWidget):
                 "Wavelet periods out of bounds!", "Value Error"
             )
             return False
-        self.T_min_value = float(T_min)
 
+        wlet_pars['T_min'] = float(T_min)
+                
         step_num = self.step_num.text()
         check, _, _ = posintV.validate(step_num, 0)
         if self.debug:
@@ -689,10 +725,22 @@ class SynthSignalGen(QWidget):
                 "The Number of periods must be a positive integer!", "Value Error"
             )
             return False
-        self.step_num_value = int(step_num)
 
+        wlet_pars['step_num'] = int(step_num)
+        if int(step_num) > 1000:
+
+            choice = QMessageBox.question(
+                self,
+                "Too much periods?: ",
+                "High number of periods: Do you want to continue?",
+                QMessageBox.Yes | QMessageBox.No,
+            )
+            if choice == QMessageBox.Yes:
+                pass
+            else:
+                return False        
+        
         text = self.T_max.text()
-
         T_max = text.replace(",", ".")
         check, _, _ = vali.validate(T_max, 0)
         if self.debug:
@@ -703,7 +751,7 @@ class SynthSignalGen(QWidget):
                 "Wavelet highest period out of bounds!", "Value Error"
             )
             return False
-        self.T_max_value = float(T_max)
+        wlet_pars['T_max'] = float(T_max)
 
         text = self.p_max.text()
         p_max = text.replace(",", ".")
@@ -714,48 +762,83 @@ class SynthSignalGen(QWidget):
 
         # check for empty string:
         if p_max:
-            self.p_max_value = float(p_max)
+            wlet_pars['p_max'] = float(p_max)
         else:
-            self.p_max_value = None
+            wlet_pars['p_max'] = None
 
+        # -- the checkboxes --
+            
+        # detrend for the analysis?
+        if self.cb_use_detrended.isChecked():
+            T_c = self.get_T_c(self.T_c_edit)
+            if T_c is None:
+                return False # abort settings            
+            wlet_pars['T_c'] = T_c
+        else:
+            # indicates no detrending requested
+            wlet_pars['T_c'] = False
+            
+        # amplitude normalization is downstram of detrending!
+        if self.cb_use_envelope.isChecked():
+            L = self.get_L(self.L_edit)        
+            if L is None:
+                return False # abort settings                        
+            wlet_pars['L'] = L
+        else:
+            # indicates no ampl. normalization
+            wlet_pars['L'] = False
+                    
         # success!
-        return True
-
+        return wlet_pars
+                
     def calc_trend(self):
 
         """ Uses maximal sinc window size """
-
-        trend = pyboat.sinc_smooth(raw_signal=self.raw_signal, T_c=self.T_c, dt=self.dt)
+        
+        T_c = self.get_T_c(self.T_c_edit)
+        if not T_c:
+            return
+        if self.debug:
+            print("Calculating trend with T_c = ", T_c)
+        
+        trend = pyboat.sinc_smooth(raw_signal=self.raw_signal, T_c = T_c, dt=self.dt)
         return trend
 
     def calc_envelope(self):
 
-        if self.L < 3:
+        L = self.get_L(self.L_edit)
+        if not L:
+            return
+        if self.debug:
+            print("Calculating envelope with L = ", L)
+            
+        if L / self.dt < 4:
             self.OutOfBounds = MessageWindow(
-                f"Minimum sliding\nwindow size is {3*self.dt}{self.time_unit} !",
+                f"Minimum sliding window size is {4*self.dt}{self.time_unit} !",
                 "Value Error",
             )
-            self.L = None
+            L = None
             return
 
-        if self.L > self.Nt:
-            maxL = self.Nt * self.dt
+        if L / self.dt > self.raw_signal.size:
+            maxL = self.raw_signal.size * self.dt
             self.OutOfBounds = MessageWindow(
-                f"Maximum sliding window\nsize is {maxL:.2f} {self.time_unit}!",
+                f"Maximum sliding window size is {maxL:.2f} {self.time_unit}!",
                 "Value Error",
             )
-            self.L = None
+            L = None
             return
 
         # cut of frequency set?!
-        if self.T_c:
-            if self.debug:
-                print("calculating envelope for detrended signal", self.L, self.T_c)
+        if self.get_T_c(self.T_c_edit):
 
             trend = self.calc_trend()
+            if trend is None:
+                return
+            
             signal = self.raw_signal - trend
             envelope = pyboat.sliding_window_amplitude(signal,
-                                                       window_size=self.L,
+                                                       window_size = L,
                                                        dt = self.dt)
 
             if self.cb_detrend.isChecked():
@@ -768,11 +851,11 @@ class SynthSignalGen(QWidget):
         # otherwise add the mean
         else:
             if self.debug:
-                print("calculating envelope for raw signal", self.L)
+                print("calculating envelope for raw signal", L)
 
             mean = self.raw_signal.mean()
             envelope = pyboat.sliding_window_amplitude(
-                self.raw_signal, window_size=self.L
+                self.raw_signal, window_size = L, dt = self.dt
             )
             return envelope + mean
 
@@ -782,6 +865,12 @@ class SynthSignalGen(QWidget):
         Retrieves all paramters from the synthesizer controls
         and calls the ssg. All line edits have validators set,
         however we have to check for intermediate empty inputs..
+
+        This sets
+
+        self.raw_signal
+
+        according to the synthesizer controls.
         """
 
         # the signal components
@@ -799,8 +888,8 @@ class SynthSignalGen(QWidget):
         if self.debug:
             print("Nt changed to:", self.Nt)
 
-        self.set_initial_periods(force=True)
-        self.set_initial_T_c(force=True)
+        self.set_initial_periods(force=False)
+        self.set_initial_T_c(force=False)
 
         T_edits = [self.T11_edit, self.T12_edit, self.T21_edit, self.T22_edit]
 
@@ -871,16 +960,31 @@ class SynthSignalGen(QWidget):
 
         # noise
         if self.noise_box.isChecked():
-            # QDoubleValidator is a screw up..
-            alpha = float(self.alpha_edit.text())
-            if not 0 < alpha < 1:
+
+
+            try:
+                # QDoubleValidator is a screw up..
+                alpha = float(self.alpha_edit.text())
+            # empty input
+            except ValueError:
                 self.OutOfBounds = MessageWindow(
-                    "AR1 parameter must be smaller than 1!", "Value Error"
+                    "Missing AR(1) alpha parameter!", "Value Error"
                 )
                 return
-            if not self.d_edit.hasAcceptableInput():
+            
+            if not 0 <= alpha < 1:
                 self.OutOfBounds = MessageWindow(
-                    "Missing noise parameters!", "Value Error"
+                    "AR1 parameter must be between 0 and <1!", "Value Error"
+                )
+                return
+
+            # Noise amplitude
+            try:
+                d = float(self.d_edit.text())
+
+            except ValueError:
+                self.OutOfBounds = MessageWindow(
+                    "Missing Noise Strength parameter!", "Value Error"
                 )
                 return
 
@@ -925,19 +1029,18 @@ class SynthSignalGen(QWidget):
             )
 
         # check if trend is needed
-        if self.T_c and (self.cb_trend.isChecked() or self.cb_detrend.isChecked()):
-            if self.debug:
-                print("Calculating trend with T_c = ", self.T_c)
+        if self.cb_trend.isChecked() or self.cb_detrend.isChecked():
             trend = self.calc_trend()
-
+            if trend is None:
+                return
         else:
             trend = None
 
         # envelope calculation
-        if self.L and self.cb_envelope.isChecked():
-            if self.debug:
-                print("Calculating envelope with L = ", self.L)
+        if self.cb_envelope.isChecked():
             envelope = self.calc_envelope()
+            if envelope is None:
+                return
 
         else:
             envelope = None
@@ -962,14 +1065,14 @@ class SynthSignalGen(QWidget):
             ax2 = pl.draw_detrended(
                 ax1, time_vector=self.tvec, detrended=self.raw_signal - trend
             )
-            ax2.legend(fontsize=pl.tick_label_size)
+            ax2.legend(fontsize=pl.tick_label_size, loc='lower left')
         if envelope is not None and not self.cb_detrend.isChecked():
             pl.draw_envelope(ax1, time_vector=self.tvec, envelope=envelope)
 
         # plot on detrended axis
         if envelope is not None and self.cb_detrend.isChecked():
             pl.draw_envelope(ax2, time_vector=self.tvec, envelope=envelope)
-            ax2.legend(fontsize=pl.tick_label_size)
+            ax2.legend(fontsize=pl.tick_label_size, loc='lower left')
         self.tsCanvas.fig1.subplots_adjust(bottom=0.15, left=0.15, right=0.85)
         # add a simple legend
         ax1.legend(fontsize=pl.tick_label_size)
@@ -978,71 +1081,45 @@ class SynthSignalGen(QWidget):
         self.tsCanvas.show()
 
     def run_wavelet_ana(self):
-        """ run the Wavelet Analysis """
+        
+        """ run the Wavelet Analysis on the synthetic signal """
 
         if not np.any(self.raw_signal):
             self.NoSignalSelected = MessageWindow(
-                "Please select a signal first!", "No Signal"
+                "Please create a signal first!", "No Signal"
             )
             return False
 
-        succ = self.set_wlet_pars()  # Error handling done there
-        if not succ:
+        wlet_pars = self.set_wlet_pars()  # Error handling done there
+        if not wlet_pars:
             if self.debug:
                 print("Wavelet parameters could not be set!")
             return False
 
-        # move to set_wlet_pars?!
-        if self.step_num_value > 1000:
-
-            choice = QMessageBox.question(
-                self,
-                "Too much periods?: ",
-                "High number of periods: Do you want to continue?",
-                QMessageBox.Yes | QMessageBox.No,
-            )
-            if choice == QMessageBox.Yes:
-                pass
-            else:
-                return
-
-        # detrend for the analysis?
-        if self.cb_use_detrended.isChecked() and not self.T_c:
-            self.NoTrend = MessageWindow(
-                "Detrending parameter not set,\n" + "specify a cut-off period!",
-                "No Trend",
-            )
-            return
-
-        elif self.cb_use_detrended.isChecked():
+        if self.cb_use_detrended.isChecked():
             trend = self.calc_trend()
             signal = self.raw_signal - trend
         else:
             signal = self.raw_signal
 
-        # amplitude normalization is downstram of detrending!
-        if self.cb_use_envelope.isChecked() and not self.L:
-            self.NoTrend = MessageWindow(
-                "Envelope parameter not set,\n" + "specify a sliding window size!",
-                "No Envelope",
-            )
-            return
 
-        elif self.cb_use_envelope.isChecked():
-            signal = pyboat.normalize_with_envelope(signal, self.L, self.dt)
+        if self.cb_use_envelope.isChecked():
+            L = self.get_L(self.L_edit)
+            signal = pyboat.normalize_with_envelope(signal, L, dt = self.dt)
 
         self.w_position += 20
 
+        
         self.anaWindows[self.w_position] = WaveletAnalyzer(
             signal=signal,
             dt=self.dt,
-            T_min=self.T_min_value,
-            T_max=self.T_max_value,
+            Tmin = wlet_pars['T_min'],
+            Tmax = wlet_pars['T_max'],
+            pow_max = wlet_pars['p_max'],
+            step_num=wlet_pars['step_num'],
             position=self.w_position,
-            signal_id="Synthetic Signal",
-            step_num=self.step_num_value,
-            p_max=self.p_max_value,
-            time_unit=self.time_unit,
+            signal_id='Synthetic Signal',
+            time_unit=self.time_unit,            
             DEBUG=self.debug,
         )
 
@@ -1056,28 +1133,24 @@ class SynthSignalGen(QWidget):
         # shift new analyser windows
         self.w_position += 20
 
-        if self.cb_use_detrended2.isChecked() and not self.T_c:
-            self.NoTrend = MessageWindow(
-                "Detrending not set, can not use detrended signal!", "No Trend"
-            )
-            return
 
-        elif self.cb_use_detrended2.isChecked():
+        if self.cb_use_detrended2.isChecked():
             trend = self.calc_trend()
-            signal = self.raw_signal - trend
+            if trend is not None:
+                signal = self.raw_signal - trend
+            else:
+                return
         else:
             signal = self.raw_signal
 
-        if self.cb_use_envelope2.isChecked() and not self.L:
-            self.NoTrend = MessageWindow(
-                "Envelope parameter not set,\n" + "specify a sliding window size!",
-                "No Envelope",
-            )
-            return
-
-        elif self.cb_use_envelope2.isChecked():
-            signal = pyboat.normalize_with_envelope(signal, self.L, self.dt)
 
+        if self.cb_use_envelope2.isChecked():
+            L = self.get_L(self.L_edit)
+            if L is not None:                
+                signal = pyboat.normalize_with_envelope(signal, L, self.dt)
+            else:
+                return
+            
         # periods or frequencies?
         if self.cb_FourierT.isChecked():
             show_T = False
